import grpc

Unary = "unary"
ClientStream = "client_stream"
ServerStream = "server_stream"
BidiStream = "bidi_stream"


def type_from_method(request_streaming, response_streaming):
    if not request_streaming and not response_streaming:
        return Unary
    elif request_streaming and not response_streaming:
        return ClientStream
    elif not request_streaming and response_streaming:
        return ServerStream
    else:
        return BidiStream


GRPC_STATUS_CODE_TO_STRING = {
    grpc.StatusCode.OK: "OK",
    grpc.StatusCode.CANCELLED: "Canceled",
    grpc.StatusCode.UNKNOWN: "Unknown",
    grpc.StatusCode.INVALID_ARGUMENT: "InvalidArgument",
    grpc.StatusCode.DEADLINE_EXCEEDED: "DeadlineExceeded",
    grpc.StatusCode.NOT_FOUND: "NotFound",
    grpc.StatusCode.ALREADY_EXISTS: "AlreadyExists",
    grpc.StatusCode.PERMISSION_DENIED: "PermissionDenied",
    grpc.StatusCode.UNAUTHENTICATED: "Unauthenticated",
    grpc.StatusCode.RESOURCE_EXHAUSTED: "ResourceExhausted",
    grpc.StatusCode.FAILED_PRECONDITION: "FailedPrecondition",
    grpc.StatusCode.ABORTED: "Aborted",
    grpc.StatusCode.OUT_OF_RANGE: "OutOfRange",
    grpc.StatusCode.UNIMPLEMENTED: "Unimplemented",
    grpc.StatusCode.INTERNAL: "Internal",
    grpc.StatusCode.UNAVAILABLE: "Unavailable",
    grpc.StatusCode.DATA_LOSS: "DataLoss",
}


def split_call_details(details, minimum_grpc_method_path_items=3):
    parts = details.method.split("/")
    if len(parts) < minimum_grpc_method_path_items:
        return '', '', False

    grpc_service, grpc_method = parts[1:minimum_grpc_method_path_items]
    return grpc_service, grpc_method, True


def code_to_string(code):
    if code is None:
        return None
    try:
        return GRPC_STATUS_CODE_TO_STRING.get(code, code.name)
    except AttributeError:
        return code.name
